@@ -6,7 +6,7 @@ require_once 'Horde/History.php';
  * Kronolith_Driver:: defines an API for implementing storage backends
  * for Kronolith.
  *
- * $Horde: kronolith/lib/Driver.php,v 1.116.2.3 2005/01/20 16:05:53 chuck Exp $
+ * $Horde: kronolith/lib/Driver.php,v 1.116.2.22 2005/10/14 13:56:10 chuck Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -16,13 +16,15 @@ class Kronolith_Driver {
 
     /**
      * A hash containing any parameters for the current driver.
-     * @var array $_params
+     *
+     * @var array
      */
     var $_params = array();
 
     /**
      * The current calendar.
-     * @var string $_calendar
+     *
+     * @var string
      */
     var $_calendar;
 
@@ -30,7 +32,7 @@ class Kronolith_Driver {
      * Constructor - just store the $params in our newly-created
      * object. All other work is done by open().
      *
-     * @param optional array $params  Any parameters needed for this driver.
+     * @param array $params  Any parameters needed for this driver.
      */
     function Kronolith_Driver($params = array())
     {
@@ -77,7 +79,8 @@ class Kronolith_Driver {
     /**
      * Search a calendar.
      *
-     * @param object $query  A Kronolith_Event object with the criteria to search for.
+     * @param object Kronolith_Event $query  A Kronolith_Event object with the
+     *                                       criteria to search for.
      *
      * @return mixed  An array of Kronolith_Events or a PEAR_Error.
      */
@@ -94,8 +97,17 @@ class Kronolith_Driver {
             return $events;
         }
 
-        $startTime = $query->start->timestamp();
-        $endTime = $query->end->timestamp();
+        if (isset($query->start)) {
+            $startTime = $query->start->timestamp();
+        } else {
+            $startTime = null;
+        }
+
+        if (isset($query->end)) {
+            $endTime = $query->end->timestamp();
+        } else {
+            $endTime = null;
+        }
 
         foreach ($events as $eventid) {
             $event = &$this->getEvent($eventid);
@@ -106,7 +118,8 @@ class Kronolith_Driver {
             $evStartTime = $event->start->timestamp();
             $evEndTime = $event->end->timestamp();
 
-            if ((($evEndTime > $startTime && $evStartTime < $endTime) ||
+            if (((($evEndTime > $startTime || !isset($startTime)) &&
+                  ($evStartTime < $endTime || !isset($endTime))) ||
                  ($event->getRecurType() != KRONOLITH_RECUR_NONE && $evEndTime >= $startTime && $evStartTime <= $endTime)) &&
                 (empty($query->title) || stristr($event->getTitle(), $query->title)) &&
                 (empty($query->location) || stristr($event->getLocation(), $query->location)) &&
@@ -129,7 +142,7 @@ class Kronolith_Driver {
      * @return Horde_Date | boolean  The date of the next recurrence or false
      *                               if the event does not recur after $afterDate.
      */
-    function &nextRecurrence($eventId, $afterDate)
+    function nextRecurrence($eventId, $afterDate)
     {
         $event = &$this->getEvent($eventId);
         if (is_a($event, 'PEAR_Error')) {
@@ -143,16 +156,14 @@ class Kronolith_Driver {
      * Attempts to return a concrete Kronolith_Driver instance based
      * on $driver.
      *
-     * @param $driver   The type of concrete Kronolith_Driver subclass to return.
-     *                  This is based on the calendar driver ($driver). The
-     *                  code is dynamically included.
+     * @param string $driver  The type of concrete Kronolith_Driver subclass
+     *                        to return.
      *
-     * @param $params   (optional) A hash containing any additional
-     *                  configuration or connection parameters a subclass
-     *                  might need.
+     * @param array $params   A hash containing any additional configuration or
+     *                        connection parameters a subclass might need.
      *
-     * @return          The newly created concrete Kronolith_Driver instance, or
-     *                  a PEAR_Error on error.
+     * @return Kronolith_Driver  The newly created concrete Kronolith_Driver
+     *                           instance, or a PEAR_Error on error.
      */
     function &factory($driver = null, $params = null)
     {
@@ -189,61 +200,71 @@ class Kronolith_Event {
     /**
      * Flag that is set to true if this event has data from either a
      * storage backend or a form or other import method.
-     * @var boolean $initialized
+     *
+     * @var boolean
      */
     var $initialized = false;
 
     /**
      * Flag that is set to true if this event exists in a storage driver.
-     * @var boolean $stored
+     *
+     * @var boolean
      */
     var $stored = false;
 
     /**
      * The driver unique identifier for this event.
-     * @var string $eventID
+     *
+     * @var string
      */
     var $eventID = null;
 
     /**
      * The UID for this event.
-     * @var string $_uid
+     *
+     * @var string
      */
     var $_uid = null;
 
     /**
-     * The user id of the creator of the event
-     * @var string $creatorID
+     * The user id of the creator of the event.
+     *
+     * @var string
      */
     var $creatorID = null;
 
     /**
-     * The title of this event,
-     * @var string $title
+     * The title of this event.
+     *
+     * @var string
      */
     var $title = '';
 
     /**
      * The identifier of the category this event belongs to.
-     * @var integer $category
+     *
+     * @var integer
      */
     var $category = 0;
 
     /**
      * The location this event occurs at.
-     * @var string $location
+     *
+     * @var string
      */
     var $location = '';
 
     /**
      * The status of this event.
-     * @var integer $status
+     *
+     * @var integer
      */
     var $status = KRONOLITH_STATUS_CONFIRMED;
 
     /**
      * The description for this event
-     * @var string $description
+     *
+     * @var string
      */
     var $description = '';
 
@@ -255,79 +276,91 @@ class Kronolith_Event {
      * keys 'attendance' and 'response' pointing to the attendees' attendance
      * and response values, respectively.
      *
-     * @var array $attendees
+     * @var array
      */
     var $attendees = array();
 
     /**
      * All the key words associtated with this event.
-     * @var array $keywords
+     *
+     * @var array
      */
     var $keywords = array();
 
     /**
      * All the exceptions from recurrence for this event.
-     * @var array $exceptions
+     *
+     * @var array
      */
     var $exceptions = array();
 
     /**
      * The start time of the event.
-     * @var Horde_Date $start
+     *
+     * @var Horde_Date
      */
     var $start;
 
     /**
      * The end time of the event.
-     * @var Horde_Date $end
+     *
+     * @var Horde_Date
      */
     var $end;
 
     /**
      * The end time of the recurrence interval.
-     * @var Horde_Date $recurEnd
+     *
+     * @var Horde_Date
      */
     var $recurEnd = null;
 
     /**
      * The duration of this event in minutes
-     * @var integer $durMin
+     *
+     * @var integer
      */
     var $durMin = 0;
 
     /**
      * Number of minutes before the event starts to trigger an alarm.
-     * @var integer $alarm
+     *
+     * @var integer
      */
     var $alarm = 0;
 
     /**
      * The type of recurrence this event follows. KRONOLITH_RECUR_* constant.
-     * @var integer $recurType
+     *
+     * @var integer
      */
     var $recurType = KRONOLITH_RECUR_NONE;
 
     /**
      * TODO The length of time between recurrences in seconds?
-     * @var integer $recurInterval
+     *
+     * @var integer
      */
     var $recurInterval = null;
 
     /**
      * Any additional recurrence data.
-     * @var integer $recurData
+     *
+     * @var integer
      */
     var $recurData = null;
 
     /**
      * The identifier of the calender this event exists on.
-     * @var string $_calendar
+     *
+     * @var string
      */
     var $_calendar;
 
     /**
      * The VarRenderer class to use for printing select elements.
-     * @var object Horde_UI_VarRenderer $_varRenderer
+     *
+     * @var Horde_UI_VarRenderer
      */
     var $_varRenderer;
 
@@ -368,8 +401,9 @@ class Kronolith_Event {
     /**
      * Export this event in iCalendar format.
      *
-     * @param Horde_iCalendar  &$calendar  A Horde_iCalendar object that acts as a container.
-     * @param Identity         $identity   The Identity object of the organizer.
+     * @param Horde_iCalendar &$calendar  A Horde_iCalendar object that acts as
+     *                                    a container.
+     * @param Identity $identity          The Identity object of the organizer.
      *
      * @return Horde_iCalendar_vevent  The vEvent object for this event.
      */
@@ -393,13 +427,51 @@ class Kronolith_Event {
         $vEvent->setAttribute('DTSTAMP', time());
         $vEvent->setAttribute('UID', $this->_uid);
         $vEvent->setAttribute('SUMMARY', $this->title);
-        $vEvent->setAttribute('DESCRIPTION', $this->description);
-        $vEvent->setAttribute('CATEGORIES', $this->getCategory());
-        $vEvent->setAttribute('LOCATION', $this->location);
         $vEvent->setAttribute('TRANSP', 'OPAQUE');
         $vEvent->setAttribute('ORGANIZER',
                               'MAILTO:' . $identity->getValue('from_addr'),
                               array('CN' => $identity->getValue('fullname')));
+        if (!empty($this->description)) {
+            $vEvent->setAttribute('DESCRIPTION', $this->description);
+        }
+        $categories = $this->getCategory();
+        if (!empty($categories)) {
+            $vEvent->setAttribute('CATEGORIES', $categories);
+        }
+        if (!empty($this->location)) {
+            $vEvent->setAttribute('LOCATION', $this->location);
+        }
+
+        // Attendees.
+        foreach ($this->getAttendees() as $email => $status) {
+            $params = array();
+            switch ($status['attendance']) {
+            case KRONOLITH_PART_REQUIRED:
+                $params['ROLE'] = 'REQ-PARTICIPANT';
+                break;
+            case KRONOLITH_PART_OPTIONAL:
+                $params['ROLE'] = 'OPT-PARTICIPANT';
+                break;
+            case KRONOLITH_PART_NONE:
+                $params['ROLE'] = 'NON-PARTICIPANT';
+                break;
+            }
+            switch ($status['response']) {
+            case KRONOLITH_RESPONSE_NONE:
+                $params['PARTSTAT'] = 'NEEDS-ACTION';
+                break;
+            case KRONOLITH_RESPONSE_ACCEPTED:
+                $params['PARTSTAT'] = 'ACCEPTED';
+                break;
+            case KRONOLITH_RESPONSE_DECLINED:
+                $params['PARTSTAT'] = 'DECLINED';
+                break;
+            case KRONOLITH_RESPONSE_TENTATIVE:
+                $params['PARTSTAT'] = 'TENTATIVE';
+                break;
+            }
+            $vEvent->setAttribute('ATTENDEE', 'MAILTO:' . $email, $params);
+        }
 
         // Recurrence.
         if ($this->recurType) {
@@ -472,10 +544,17 @@ class Kronolith_Event {
      * Update the properties of this event from a
      * Horde_iCalendar_vevent object.
      *
-     * @param Horde_iCalendar_vevent $vEvent  The iCalendar data to update from.
+     * @param Horde_iCalendar_vevent $vEvent  The iCalendar data to update
+     *                                        from.
      */
     function fromiCalendar($vEvent)
     {
+        // Unique ID.
+        $uid = $vEvent->getAttribute('UID');
+        if (!empty($uid) && !is_a($uid, 'PEAR_Error')) {
+            $this->setUID($uid);
+        }
+
         // Title, category and description.
         $title = $vEvent->getAttribute('SUMMARY');
         if (!is_array($title) && !is_a($title, 'PEAR_Error')) {
@@ -589,6 +668,13 @@ class Kronolith_Event {
                     $this->setRecurType(KRONOLITH_RECUR_YEARLY);
                     break;
                 }
+
+                if (isset($rdata['UNTIL'])) {
+                    list($year, $month, $mday) = sscanf($rdata['UNTIL'], '%04d%02d%02d');
+                    $this->recurEnd = &new Horde_Date(array('year' => $year,
+                                                            'month' => $month,
+                                                            'mday' => $mday));
+                }
             } else {
                 // No recurrence data - event does not recur.
                 $this->setRecurType(KRONOLITH_RECUR_NONE);
@@ -623,6 +709,8 @@ class Kronolith_Event {
         }
         if (!empty($hash['title'])) {
             $this->setTitle($hash['title']);
+        } else {
+            return PEAR::raiseError(_("Events must have a title."));
         }
         if (!empty($hash['description'])) {
             $this->setDescription($hash['description']);
@@ -655,6 +743,11 @@ class Kronolith_Event {
                 $this->start = &new Horde_Date(array('year' => $date[0], 'month' => $date[1], 'mday' => $date[2],
                                                      'hour' => $time[0], 'min' => $time[1], 'sec' => $time[2]));
             }
+        } else {
+            return PEAR::raiseError(_("Events must have a start date."));
+        }
+        if (empty($hash['duration']) && empty($hash['end_date'])) {
+            return PEAR::raiseError(_("Events must have an end date or a duration."));
         }
         if (!empty($hash['duration'])) {
             $weeks = str_replace('W', '', $hash['duration'][1]);
@@ -718,7 +811,7 @@ class Kronolith_Event {
     function save()
     {
         if (!$this->isInitialized()) {
-            return PEAR::raiseError('no data');
+            return PEAR::raiseError('Event not yet initialized');
         }
 
         $this->toDriver();
@@ -729,9 +822,9 @@ class Kronolith_Event {
     /**
      * Add an exception to a recurring event.
      *
-     * @param integer $year     The year of the execption.
-     * @param integer $month    The month of the execption.
-     * @param integer $mday     The day of the month of the exception.
+     * @param integer $year   The year of the execption.
+     * @param integer $month  The month of the execption.
+     * @param integer $mday   The day of the month of the exception.
      */
     function addException($year, $month, $mday)
     {
@@ -741,9 +834,9 @@ class Kronolith_Event {
     /**
      * Check if an exception exists for a given reccurence of an event.
      *
-     * @param integer $year     The year of the reucrance.
-     * @param integer $month    The month of the reucrance.
-     * @param integer $mday     The day of the month of the reucrance.
+     * @param integer $year   The year of the reucrance.
+     * @param integer $month  The month of the reucrance.
+     * @param integer $mday   The day of the month of the reucrance.
      *
      * @return boolean  True if an exception exists for the given date.
      */
@@ -753,10 +846,10 @@ class Kronolith_Event {
     }
 
     /**
-     * Retrieve all the exceptions for this event
+     * Retrieve all the exceptions for this event.
      *
-     * @return array    Array containing the dates of all the exceptions in
-     *                  YYYYMMDD form.
+     * @return array  Array containing the dates of all the exceptions in
+     *                YYYYMMDD form.
      */
     function getExceptions()
     {
@@ -804,7 +897,7 @@ class Kronolith_Event {
     /**
      * Return the days this event recurs on.
      *
-     * @return intenger A mask specifying the day(s) this event recurs on.
+     * @return integer  A mask specifying the day(s) this event recurs on.
      */
     function getRecurOnDays()
     {
@@ -829,7 +922,7 @@ class Kronolith_Event {
     /**
      * Set the length of time between recurrences of this event.
      *
-     * @param integer $interval     The number of seconds between recurrences.
+     * @param integer $interval  The number of seconds between recurrences.
      */
     function setRecurInterval($interval)
     {
@@ -892,9 +985,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Retrieve the id of the user who created the event
+     * Retrieve the id of the user who created the event.
      *
-     * @return string The creator id
+     * @return string  The creator id
      */
     function getCreatorId()
     {
@@ -902,9 +995,9 @@ class Kronolith_Event {
     }
 
     /**
-     * Set the id of the creator of the event
+     * Set the id of the creator of the event.
      *
-     * @param string $creatorID The user id for who created the event
+     * @param string $creatorID  The user id for the user who created the event
      */
     function setCreatorId($creatorID)
     {
@@ -926,8 +1019,9 @@ class Kronolith_Event {
             return '';
         }
 
-        $share = $GLOBALS['all_calendars'][$this->getCalendar()];
-        if (!is_a($share, 'PEAR_Error') && $share->hasPermission(Auth::getAuth(), PERMS_READ, $this->getCreatorId())) {
+        if (isset($GLOBALS['all_calendars'][$this->getCalendar()]) &&
+            !is_a($share = $GLOBALS['all_calendars'][$this->getCalendar()], 'PEAR_Error') &&
+            $share->hasPermission(Auth::getAuth(), PERMS_READ, $this->getCreatorId())) {
             return !empty($this->title) ? $this->title : _("[Unnamed event]");
         } else {
             global $prefs;
@@ -950,7 +1044,7 @@ class Kronolith_Event {
     /**
      * Retieve the description of this event.
      *
-     * @return string The description of this event.
+     * @return string  The description of this event.
      */
     function getDescription()
     {
@@ -960,7 +1054,7 @@ class Kronolith_Event {
     /**
      * Set the description of this event.
      *
-     * @param string $description The new description for this event.
+     * @param string $description  The new description for this event.
      */
     function setDescription($description)
     {
@@ -970,8 +1064,8 @@ class Kronolith_Event {
     /**
      * Set the category this event belongs to.
      *
-     * @param integer $category     The identifier of the category this
-     *                                  event belongs to.
+     * @param integer $category  The identifier of the category this event
+     *                           belongs to.
      */
     function setCategory($category)
     {
@@ -981,7 +1075,7 @@ class Kronolith_Event {
     /**
      * Retrieve the category this event belongs to.
      *
-     * @return integer The identifier of the category this event belongs to.
+     * @return integer  The identifier of the category this event belongs to.
      */
     function getCategory()
     {
@@ -1001,7 +1095,7 @@ class Kronolith_Event {
     /**
      * Retrieve the location this event occurs at.
      *
-     * @return string   The location of this event.
+     * @return string  The location of this event.
      */
     function getLocation()
     {
@@ -1011,7 +1105,7 @@ class Kronolith_Event {
     /**
      * Retrieve the event status.
      *
-     * @return integer   The status of this event.
+     * @return integer  The status of this event.
      */
     function getStatus()
     {
@@ -1023,7 +1117,7 @@ class Kronolith_Event {
      *
      * @param integer $status  The status value to check against.
      *
-     * @return boolean   True if the events status is the same as $status.
+     * @return boolean  True if the events status is the same as $status.
      */
     function hasStatus($status)
     {
@@ -1043,8 +1137,8 @@ class Kronolith_Event {
     /**
      * Sets the entire attendee array.
      *
-     * @param array $attendees   The new attendees array. This should be of
-     *                           the correct format to avoid driver problems.
+     * @param array $attendees  The new attendees array. This should be of the
+     *                          correct format to avoid driver problems.
      */
     function setAttendees($attendees)
     {
@@ -1078,7 +1172,7 @@ class Kronolith_Event {
     /**
      * Removes the specified attendee from the current event.
      *
-     * @param string $email        The email address of the attendee.
+     * @param string $email  The email address of the attendee.
      */
     function removeAttendee($email)
     {
@@ -1101,9 +1195,10 @@ class Kronolith_Event {
      * Checks to see whether the specified attendee is associated with the
      * current event.
      *
-     * @param string $email        The email address of the attendee.
+     * @param string $email  The email address of the attendee.
      *
-     * @return boolen  True if the specified attendee is present for this event.
+     * @return boolean  True if the specified attendee is present for this
+     *                  event.
      */
     function hasAttendee($email)
     {
@@ -1340,7 +1435,7 @@ class Kronolith_Event {
 
     function getDuration()
     {
-        static $duration;
+        static $duration = null;
         if (isset($duration)) {
             return $duration;
         }
@@ -1373,7 +1468,7 @@ class Kronolith_Event {
             $whole_day_match = false;
         }
 
-        $duration = &new stdClass();
+        $duration = new stdClass;
         $duration->day = $dur_day_match;
         $duration->hour = $dur_hour_match;
         $duration->min = $dur_min_match;
@@ -1427,7 +1522,7 @@ class Kronolith_Event {
             break;
 
         case 'start_min':
-            $sel = $this->start->min;
+            $sel = sprintf('%02d', $this->start->min);
             for ($i = 0; $i < 12; $i++) {
                 $min = sprintf('%02d', $i * 5);
                 $options[$min] = $min;
@@ -1474,6 +1569,7 @@ class Kronolith_Event {
 
         case 'end_min':
             $sel = $this->isInitialized() ? $this->end->min : $this->start->min;
+            $sel = sprintf('%02d', $sel);
             for ($i = 0; $i < 12; $i++) {
                 $min = sprintf('%02d', $i * 5);
                 $options[$min] = $min;
@@ -1593,7 +1689,7 @@ class Kronolith_Event {
         $share = $GLOBALS['all_calendars'][$this->getCalendar()];
         if (!is_a($share, 'PEAR_Error') && $share->hasPermission(Auth::getAuth(), PERMS_READ, $this->getCreatorId())) {
             if (isset($this->remoteCal)) {
-                $url = Util::addParameter('viewevent.php', 'eventID', $this->eventIndex);
+                $url = Util::addParameter('viewevent.php', 'eventID', $this->eventID);
                 $url = Util::addParameter($url, 'calendar', '**remote');
                 $url = Util::addParameter($url, 'remoteCal', $this->remoteCal);
                 $url = Util::addParameter($url, 'timestamp', $timestamp);
@@ -1687,15 +1783,15 @@ class Kronolith_Event {
                         _("Alarm 1 minute before") :
                         sprintf(_("Alarm %d minutes before"), $alarm_value);
                 }
-                $status .= Horde::img('alarm_small.png', $title, 'align="middle"');
+                $status .= Horde::img('alarm_small.png', $title);
             }
 
             if (!$this->hasRecurType(KRONOLITH_RECUR_NONE)) {
-                $status .= Horde::img('recur.png', Kronolith::recurToString($this->recurType), 'align="middle"');
+                $status .= Horde::img('recur.png', Kronolith::recurToString($this->recurType));
             }
 
             if (!empty($this->attendees)) {
-                $status .= Horde::img('attendees.png', count($this->attendees) == 1 ? _("1 attendee") : sprintf(_("%s attendees"), count($this->attendees)), 'align="middle"');
+                $status .= Horde::img('attendees.png', count($this->attendees) == 1 ? _("1 attendee") : sprintf(_("%s attendees"), count($this->attendees)));
             }
 
             if (!empty($status)) {
@@ -1703,23 +1799,29 @@ class Kronolith_Event {
             }
 
             if (!$print_view) {
-                if (!is_a($share, 'PEAR_Error') && $share->hasPermission(Auth::getAuth(), PERMS_DELETE, $this->getCreatorId())) {
-                    if (isset($this->eventID)) {
-                        $url = Util::addParameter('editevent.php', 'eventID', $this->eventID);
-                        $url = Util::addParameter($url, 'calendar', $this->getCalendar());
-                        $url = Util::addParameter($url, 'timestamp', $timestamp);
-                        $url = Util::addParameter($url, 'url', str_replace('&amp;', '&', Horde::selfUrl(true, false)));
-                        $link .= ' ' . Horde::link(Horde::applicationUrl($url), sprintf(_("Edit %s"), $this->title)) . Horde::img('edit-small.png', _("Edit"), array('title' => sprintf(_("Edit %s"), $this->getTitle()), 'align' => 'middle'), $registry->getImageDir('horde')) . '</a>';
-
-                        $url = Util::addParameter('delevent.php', 'eventID', $this->eventID);
-                        $url = Util::addParameter($url, 'calendar', $this->getCalendar());
-                        $url = Util::addParameter($url, 'timestamp', $timestamp);
-                        $url = Util::addParameter($url, 'url', str_replace('&amp;', '&', Horde::selfUrl(true, false)));
-                        $link .= ' ' . Horde::link(Horde::applicationUrl($url), sprintf(_("Delete %s"), $this->title)) . Horde::img('delete-small.png', _("Delete"), array('title' => sprintf(_("Delete %s"), $this->getTitle()), 'align' => 'middle'), $registry->getImageDir('horde')) . '</a>';
+                if (!is_a($share, 'PEAR_Error') &&
+                    isset($this->eventID)) {
+                    if (isset($this->remoteCal) || $share->hasPermission(Auth::getAuth(), PERMS_EDIT, $this->getCreatorId())) {
+                        $url = Util::addParameter('editevent.php', array('eventID' => $this->eventID,
+                                                                         'calendar' => isset($this->remoteCal) ? '**remote' : $this->getCalendar(),
+                                                                         'timestamp' => $timestamp,
+                                                                         'url' => str_replace('&amp;', '&', Horde::selfUrl(true, false))));
+                        if (isset($this->remoteCal)) {
+                            $url = Util::addParameter($url, 'remoteCal', $this->remoteCal);
+                        }
+                        $link .= ' ' . Horde::link(Horde::applicationUrl($url), sprintf(_("Edit %s"), $this->title)) . Horde::img('edit-small.png', _("Edit"), array('title' => sprintf(_("Edit %s"), $this->getTitle())), $registry->getImageDir('horde')) . '</a>';
+                    }
+                    if (!isset($this->remoteCal) && $share->hasPermission(Auth::getAuth(), PERMS_DELETE, $this->getCreatorId())) {
+                        $url = Util::addParameter('delevent.php', array('eventID' => $this->eventID,
+                                                                        'calendar' => $this->getCalendar(),
+                                                                        'timestamp' => $timestamp,
+                                                                        'url' => str_replace('&amp;', '&', Horde::selfUrl(true, false))));
+                        $link .= ' ' . Horde::link(Horde::applicationUrl($url), sprintf(_("Delete %s"), $this->title)) . Horde::img('delete-small.png', _("Delete"), array('title' => sprintf(_("Delete %s"), $this->getTitle())), $registry->getImageDir('horde')) . '</a>';
                     }
                 }
             }
         }
+
         return $link;
     }
 
@@ -1728,8 +1830,8 @@ class Kronolith_Event {
      *
      * @param Horde_Date $afterDate  Return events after this date.
      *
-     * @return Horde_Date | boolean  The date of the next recurrence or false
-     *                               if the event does not recur after $afterDate.
+     * @return Horde_Date|boolean  The date of the next recurrence or false
+     *                             if the event does not recur after $afterDate.
      */
     function nextRecurrence($afterDate)
     {
@@ -1737,7 +1839,11 @@ class Kronolith_Event {
         $after->correct();
 
         if ($this->start->compareDate($after) >= 0) {
-            return $date = &new Horde_Date($this->start);
+            return new Horde_Date($this->start);
+        }
+
+        if ($this->recurInterval == 0) {
+            return false;
         }
 
         if ($this->hasRecurEnd()) {
@@ -1755,7 +1861,7 @@ class Kronolith_Event {
             if ((!$this->hasRecurEnd() ||
                  $next->compareDate($this->recurEnd) <= 0) &&
                 $next->compareDate($after) >= 0) {
-                return $date = &new Horde_Date($next);
+                return new Horde_Date($next);
             }
             break;
 
