@@ -30,10 +30,9 @@ define('KRONOLITH_ITIP_CANCEL',  2);
 define('KRONOLITH_ERROR_FB_NOT_FOUND', 1);
 
 /**
- * The Kronolith:: class provides functionality common to all of
- * Kronolith.
+ * The Kronolith:: class provides functionality common to all of Kronolith.
  *
- * $Horde: kronolith/lib/Kronolith.php,v 1.263.2.3 2005/02/02 13:23:14 jan Exp $
+ * $Horde: kronolith/lib/Kronolith.php,v 1.263.2.18 2005/08/05 14:33:53 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Kronolith 0.1
@@ -42,12 +41,11 @@ define('KRONOLITH_ERROR_FB_NOT_FOUND', 1);
 class Kronolith {
 
     /**
-     * Returns all the events that happen each day within a time
-     * period.
+     * Returns all the events that happen each day within a time period.
      *
-     * @param object $startDate    The start of the time range.
-     * @param object $endDate      The end of the time range.
-     * @param array  $calendars    The calendars to check for events.
+     * @param object $startDate  The start of the time range.
+     * @param object $endDate    The end of the time range.
+     * @param array  $calendars  The calendars to check for events.
      *
      * @return array  The events happening in this time period.
      */
@@ -136,7 +134,7 @@ class Kronolith {
     }
 
     /**
-     * Fetch a remote calendar into the session and return the data.
+     * Fetches a remote calendar into the session and return the data.
      *
      * @param string $url  The location of the remote calendar.
      *
@@ -144,6 +142,11 @@ class Kronolith {
      */
     function getRemoteCalendar($url)
     {
+        // Treat webcal:// URLs as http://.
+        if (substr($url, 0, 9) == 'webcal://') {
+            $url = str_replace('webcal://', 'http://', $url);
+        }
+
         if (empty($_SESSION['kronolith']['remote'][$url])) {
             $options['method'] = 'GET';
             $options['timeout'] = 5;
@@ -154,14 +157,16 @@ class Kronolith {
             @$http->sendRequest();
             if ($http->getResponseCode() != 200) {
                 Horde::logMessage(sprintf('Failed to retrieve remote calendar: url = "%s", status = %s',
-                                          $url, $http->getResponseCode()), __FILE__, __LINE__, PEAR_LOG_ERR);
+                                          $url, $http->getResponseCode()),
+                                  __FILE__, __LINE__, PEAR_LOG_ERR);
                 return PEAR::raiseError(sprintf(_("Could not open %s."), $url));
             }
             $_SESSION['kronolith']['remote'][$url] = $http->getResponseBody();
 
             /* Log fetch at DEBUG level */
             Horde::logMessage(sprintf('Retrieved remote calendar for %s: url = "%s"',
-                                      Auth::getAuth(), $url), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+                                      Auth::getAuth(), $url),
+                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
         }
 
         return $_SESSION['kronolith']['remote'][$url];
@@ -196,7 +201,7 @@ class Kronolith {
                 $event = &$kronolith->getEvent();
                 $event->fromiCalendar($component);
                 $event->remoteCal = $url;
-                $event->eventIndex = $i;
+                $event->eventID = $i;
                 $events[] = $event;
             }
         }
@@ -207,13 +212,13 @@ class Kronolith {
     /**
      * Returns an event object for an event on a remote calendar.
      *
-     * This is kind of a temorary solution until we can have multiple
-     * drivers in use at the same time.
+     * This is kind of a temorary solution until we can have multiple drivers
+     * in use at the same time.
      *
      * @param $url      The url of the remote calendar.
-     * @param $eventId  The index of the event on the remote calenar.
+     * @param $eventId  The index of the event on the remote calendar.
      *
-     * @return object Kronolith_Event   The event object.
+     * @return Kronolith_Event  The event object.
      */
     function getRemoteEventObject($url, $eventId)
     {
@@ -231,11 +236,12 @@ class Kronolith {
         }
 
         $components = $iCal->getComponents();
-        if (isset($components[$eventId]) && $components[$eventId]->getType() == 'vEvent') {
+        if (isset($components[$eventId]) &&
+            $components[$eventId]->getType() == 'vEvent') {
             $event = &$kronolith->getEvent();
             $event->fromiCalendar($components[$eventId]);
             $event->remoteCal = $url;
-            $event->eventIndex = $eventId;
+            $event->eventID = $eventId;
 
             return $event;
         }
@@ -256,7 +262,8 @@ class Kronolith {
      *
      * @return array  The events happening in this time period.
      */
-    function listEvents($startDate = null, $endDate = null, $calendars = null, $showRecurrence = true)
+    function listEvents($startDate = null, $endDate = null, $calendars = null,
+                        $showRecurrence = true)
     {
         global $kronolith, $prefs, $registry;
 
@@ -276,9 +283,11 @@ class Kronolith {
 
         $eventIds = Kronolith::listEventIds($startDate, $endDate, $calendars);
 
-        $startOfPeriodTimestamp = mktime(0, 0, 0, $startDate->month, $startDate->mday, $startDate->year);
-        $endOfPeriodTimestamp = mktime(23, 59, 59, $endDate->month, $endDate->mday, $endDate->year);
-        $daysInPeriod = Date_Calc::dateDiff($startDate->mday, $startDate->month, $startDate->year, $endDate->mday, $endDate->month, $endDate->year);
+        $startOfPeriod = Util::cloneObject($startDate);
+        $startOfPeriod->hour = $startOfPeriod->min = $startOfPeriod->sec = 0;
+        $endOfPeriod = Util::cloneObject($endDate);
+        $endOfPeriod->hour = 23;
+        $endOfPeriod->min = $endOfPeriod->sec = 59;
 
         $results = array();
         foreach ($eventIds as $cal => $events) {
@@ -288,31 +297,40 @@ class Kronolith {
             }
             foreach ($events as $id) {
                 // We MUST fetch each event right before getting its
-                // recurrences; this is due to the way MCAL
-                // works. MCAL's nextRecurrence() function gives you
-                // the next recurrence for the event most recently
-                // fetched. So if you fetch all events and then loop
-                // through them, every recurrence you get will be for
-                // the last event that you fetched.
+                // recurrences; this is due to the way MCAL works. MCAL's
+                // nextRecurrence() function gives you the next recurrence for
+                // the event most recently fetched. So if you fetch all events
+                // and then loop through them, every recurrence you get will
+                // be for the last event that you fetched.
                 $event = &$kronolith->getEvent($id);
 
-                Kronolith::_getEvents($results, $event, $startDate, $endDate, $startOfPeriodTimestamp, $endOfPeriodTimestamp, $showRecurrence);
+                Kronolith::_getEvents($results, $event, $startDate, $endDate,
+                                      $startOfPeriod, $endOfPeriod,
+                                      $showRecurrence);
             }
         }
 
         /* Nag Tasks. */
-        if ($prefs->getValue('show_tasks') && $registry->hasMethod('tasks/listTasks')) {
+        if ($prefs->getValue('show_tasks') &&
+            $registry->hasMethod('tasks/listTasks')) {
             $taskList = $registry->call('tasks/listTasks');
             if (!is_a($taskList, 'PEAR_Error')) {
                 $kronolith->open(Kronolith::getDefaultCalendar(PERMS_SHOW));
-                $dueEndStamp = mktime(0, 0, 0, $endDate->month, $endDate->mday + 1, $endDate->year);
+                $dueEndStamp = mktime(0, 0, 0,
+                                      $endDate->month,
+                                      $endDate->mday + 1,
+                                      $endDate->year);
                 foreach ($taskList as $task) {
-                    if ($task['due'] >= $startOfPeriodTimestamp && $task['due'] < $dueEndStamp && empty($task['completed'])) {
+                    if (!empty($task['due']) &&
+                        $task['due'] >= $startOfPeriod->timestamp() &&
+                        $task['due'] < $dueEndStamp &&
+                        empty($task['completed'])) {
                         $event = &$kronolith->getEvent();
                         $event->setTitle(sprintf(_("Due: %s"), $task['name']));
                         $event->taskID = $task['task_id'];
                         $event->tasklistID = $task['tasklist_id'];
-                        if ($prefs->getValue('show_task_colors') && isset($task['category'])) {
+                        if ($prefs->getValue('show_task_colors') &&
+                            isset($task['category'])) {
                             $event->category = isset($task['category']) ? $task['category'] : null;
                         }
                         $event->start = &new Horde_Date($task['due']);
@@ -333,7 +351,9 @@ class Kronolith {
             if (!is_a($events, 'PEAR_Error')) {
                 $kronolith->open(Kronolith::getDefaultCalendar(PERMS_SHOW));
                 foreach ($events as $event) {
-                    Kronolith::_getEvents($results, $event, $startDate, $endDate, $startOfPeriodTimestamp, $endOfPeriodTimestamp, $showRecurrence);
+                    Kronolith::_getEvents($results, $event, $startDate,
+                                          $endDate, $startOfPeriod,
+                                          $endOfPeriod, $showRecurrence);
                 }
             }
         }
@@ -349,38 +369,40 @@ class Kronolith {
     }
 
     /**
-     * Returns recurrences of an event during a certain period.
+     * Calculates recurrences of an event during a certain period.
      *
      * @access private
-     *
-     * @return array  The list of recurrences.
      */
-    function &_getEvents(&$results, &$event, $startDate, $endDate,
-                         $startOfPeriodTimestamp, $endOfPeriodTimestamp,
-                         $showRecurrence)
+    function _getEvents(&$results, &$event, $startDate, $endDate,
+                        $startOfPeriod, $endOfPeriod,
+                        $showRecurrence)
     {
         global $kronolith;
 
         if (!$event->hasRecurType(KRONOLITH_RECUR_NONE) && $showRecurrence) {
-            /* Recurring Event */
+            /* Recurring Event. */
 
-            if ($event->start->timestamp() < $startOfPeriodTimestamp) {
-                // The first time the event happens was before the
-                // period started. Start searching for recurrences
-                // from the start of the period.
-                $next = array('year' => $startDate->year, 'month' => $startDate->month, 'mday' => $startDate->mday);
+            if ($event->start->compareDateTime($startOfPeriod) < 0) {
+                // The first time the event happens was before the period
+                // started. Start searching for recurrences from the start of
+                // the period.
+                $next = array('year' => $startDate->year,
+                              'month' => $startDate->month,
+                              'mday' => $startDate->mday);
             } else {
-                // The first time the event happens is in the
-                // range; unless there is an exception for
-                // this ocurrence, add it.
+                // The first time the event happens is in the range; unless
+                // there is an exception for this ocurrence, add it.
                 if (!$event->hasException($event->start->year,
                                           $event->start->month,
                                           $event->start->mday)) {
-                    $results[mktime(0, 0, 0, $event->start->month, $event->start->mday, $event->start->year)][$event->getID()] = $event;
+                    $results[mktime(0, 0, 0,
+                                    $event->start->month,
+                                    $event->start->mday,
+                                    $event->start->year)][$event->getID()] = $event;
                 }
 
-                // Start searching for recurrences from the
-                // day after it starts.
+                // Start searching for recurrences from the day after it
+                // starts.
                 $next = Util::cloneObject($event->start);
                 $next->mday++;
                 $next->correct();
@@ -403,61 +425,64 @@ class Kronolith {
             /* Event only occurs once. */
 
             // Work out what day it starts on.
-            if ($event->start->timestamp() < $startOfPeriodTimestamp) {
+            if ($event->start->compareDateTime($startOfPeriod) < 0) {
                 // It started before the beginning of the period.
-                $eventStartStamp = $startOfPeriodTimestamp;
+                $eventStart = Util::cloneObject($startOfPeriod);
             } else {
-                $eventStartStamp = $event->start->timestamp();
+                $eventStart = Util::cloneObject($event->start);
             }
 
             // Work out what day it ends on.
-            if ($event->end->timestamp() >= $endOfPeriodTimestamp) {
+            if ($event->end->compareDateTime($endOfPeriod) > 0) {
                 // Ends after the end of the period.
-                $eventEndStamp = $endOfPeriodTimestamp;
+                $eventEnd = Util::cloneObject($endOfPeriod);
             } else {
-                // If the event doesn't end at 12am set the
-                // end date to the current end date. If it
-                // ends at 12am and does not end at the same
-                // time that it starts (0 duration), set the
-                // end date to the previous day's end date.
+                // If the event doesn't end at 12am set the end date to the
+                // current end date. If it ends at 12am and does not end at
+                // the same time that it starts (0 duration), set the end date
+                // to the previous day's end date.
                 if ($event->end->hour != 0 ||
                     $event->end->min != 0 ||
-                    $event->start->timestamp() == $event->end->timestamp()) {
-                    $eventEndStamp = $event->end->timestamp();
+                    $event->start->compareDateTime($event->end) == 0) {
+                    $eventEnd = Util::cloneObject($event->end);
                 } else {
-                    $eventEndStamp = mktime(23, 59, 59,
-                                            $event->end->month,
-                                            $event->end->mday - 1,
-                                            $event->end->year);
+                    $eventEnd = &new Horde_Date(
+                        array('hour' => 23,
+                              'min' => 59,
+                              'sec' => 59,
+                              'month' =>$event->end->month,
+                              'mday' => $event->end->mday - 1,
+                              'year' => $event->end->year));
                 }
             }
 
             // Add the event to all the days it covers.
-            $i = date('j', $eventStartStamp);
-            $loopStamp = mktime(0, 0, 0,
-                                date('n', $eventStartStamp),
-                                $i,
-                                date('Y', $eventStartStamp));
-            while ($loopStamp <= $eventEndStamp) {
-                if (!($event->isAllDay() && $loopStamp == $eventEndStamp)) {
-                    $results[$loopStamp][$event->getID()] = $event;
+            $i = $eventStart->mday;
+            $loopDate = &new Horde_Date(array('month' => $eventStart->month,
+                                              'mday' => $i,
+                                              'year' => $eventStart->year));
+            while ($loopDate->compareDateTime($eventEnd) <= 0) {
+                if (!($event->isAllDay() &&
+                      $loopDate->compareDateTime($eventEnd) == 0)) {
+                    $results[$loopDate->timestamp()][$event->getID()] = $event;
                 }
-                $loopStamp = mktime(0, 0, 0,
-                                    date('n', $eventStartStamp),
-                                    ++$i,
-                                    date('Y', $eventStartStamp));
+                $loopDate = &new Horde_Date(
+                    array('month' => $eventStart->month,
+                          'mday' => ++$i,
+                          'year' => $eventStart->year));
+                $loopDate->correct();
             }
         }
     }
 
     /**
-     * Maps a Kronolith recurrence value to a translated string
-     * suitable for display.
+     * Maps a Kronolith recurrence value to a translated string suitable for
+     * display.
      *
-     * @param integer $type   The recurrence value; one of the
-     *                        KRONOLITH_RECUR_XXX constants.
+     * @param integer $type  The recurrence value; one of the
+     *                       KRONOLITH_RECUR_XXX constants.
      *
-     * @return string   The translated displayable recurrence value string.
+     * @return string  The translated displayable recurrence value string.
      */
     function recurToString($type)
     {
@@ -481,13 +506,13 @@ class Kronolith {
     }
 
     /**
-     * Maps a Kronolith meeting status string to a translated string
-     * suitable for display.
+     * Maps a Kronolith meeting status string to a translated string suitable
+     * for display.
      *
-     * @param integer $status   The meeting status; one of the
-     *                          KRONOLITH_STATUS_XXX constants.
+     * @param integer $status  The meeting status; one of the
+     *                         KRONOLITH_STATUS_XXX constants.
      *
-     * @return string   The translated displayable meeting status string.
+     * @return string  The translated displayable meeting status string.
      */
     function statusToString($status)
     {
@@ -505,13 +530,13 @@ class Kronolith {
     }
 
     /**
-     * Maps a Kronolith attendee response string to a translated
-     * string suitable for display.
+     * Maps a Kronolith attendee response string to a translated string
+     * suitable for display.
      *
      * @param integer $response  The attendee response; one of the
      *                           KRONOLITH_RESPONSE_XXX constants.
      *
-     * @return string   The translated displayable attendee response string.
+     * @return string  The translated displayable attendee response string.
      */
     function responseToString($response)
     {
@@ -532,13 +557,14 @@ class Kronolith {
     }
 
     /**
-     * Maps a Kronolith attendee participation string to a translated
-     * string suitable for display.
+     * Maps a Kronolith attendee participation string to a translated string
+     * suitable for display.
      *
-     * @param integer $part     The attendee participation; one of the
-     *                          KRONOLITH_PART_XXX constants.
+     * @param integer $part  The attendee participation; one of the
+     *                       KRONOLITH_PART_XXX constants.
      *
-     * @return string   The translated displayable attendee participation string.
+     * @return string  The translated displayable attendee participation
+     *                 string.
      */
     function partToString($part)
     {
@@ -559,9 +585,9 @@ class Kronolith {
      * Maps an iCalendar attendee response string to the corresponding
      * Kronolith value.
      *
-     * @param string $response   The attendee response.
+     * @param string $response  The attendee response.
      *
-     * @return string   The Kronolith response value.
+     * @return string  The Kronolith response value.
      */
     function responseFromICal($response)
     {
@@ -584,13 +610,14 @@ class Kronolith {
     /**
      * Builds the HTML for an event status widget.
      *
-     * @param string  $name     The name of the widget.
-     * @param string  $current  (optional) The selected status value.
-     * @param string  $any      (optional) Whether an 'any' item should be added
+     * @param string $name     The name of the widget.
+     * @param string $current  The selected status value.
+     * @param string $any      Whether an 'any' item should be added
      *
-     * @return string       The HTML <select> widget.
+     * @return string  The HTML <select> widget.
      */
-    function buildStatusWidget($name, $current = KRONOLITH_STATUS_CONFIRMED, $any = false)
+    function buildStatusWidget($name, $current = KRONOLITH_STATUS_CONFIRMED,
+                               $any = false)
     {
         $html = "<select id=\"$name\" name=\"$name\">";
 
@@ -621,12 +648,12 @@ class Kronolith {
     }
 
     /**
-     * List all calendars a user has access to, according to several
+     * Returns all calendars a user has access to, according to several
      * parameters/permission levels.
      *
-     * @param optional boolean $owneronly  Only return calenders that this
-     *                                     user owns? Defaults to false.
-     * @param optional integer $permission The permission to filter calendars by.
+     * @param boolean $owneronly   Only return calenders that this user owns?
+     *                             Defaults to false.
+     * @param integer $permission  The permission to filter calendars by.
      *
      * @return array  The calendar list.
      */
@@ -642,7 +669,7 @@ class Kronolith {
     }
 
     /**
-     * Get the default calendar for the current user at the specified
+     * Returns the default calendar for the current user at the specified
      * permissions level.
      */
     function getDefaultCalendar($permission = PERMS_SHOW)
@@ -664,9 +691,9 @@ class Kronolith {
     }
 
     /**
-     * Calculate the border (darker) version of a color.
+     * Calculates the border (darker) version of a color.
      *
-     * @param string $color   An HTML color, e.g.: #ffffcc.
+     * @param string $color  An HTML color, e.g.: #ffffcc.
      *
      * @return string  A darker html color.
      */
@@ -676,9 +703,9 @@ class Kronolith {
     }
 
     /**
-     * Calculate the highlight (lighter) version of a color.
+     * Calculates the highlight (lighter) version of a color.
      *
-     * @param string $color   An HTML color, e.g.: #ffffcc.
+     * @param string $color  An HTML color, e.g.: #ffffcc.
      *
      * @return string  A lighter html color.
      */
@@ -688,21 +715,19 @@ class Kronolith {
     }
 
     /**
-     * Generate the free/busy text for $calendar. Cache it for at
-     * least an hour, as well.
+     * Generates the free/busy text for $calendar. Cache it for at least an
+     * hour, as well.
      *
-     * @access public
-     *
-     * @param string | $calendar    The calendar to view free/busy slots for.
-     *        array
-     * @param integer  $startstamp  The start of the time period to retrieve.
-     * @param integer  $endstamp    The end of the time period to retrieve.
-     * @param boolean  $returnObj   (optional) Default false. Return a vFreebusy
-     *                              object instead of text.
+     * @param string|array $calendar  The calendar to view free/busy slots for.
+     * @param integer $startstamp     The start of the time period to retrieve.
+     * @param integer $endstamp       The end of the time period to retrieve.
+     * @param boolean $returnObj      Default false. Return a vFreebusy object
+     *                                instead of text.
      *
      * @return string  The free/busy text.
      */
-    function generateFreeBusy($calendar, $startstamp = null, $endstamp = null, $returnObj = false)
+    function generateFreeBusy($calendar, $startstamp = null, $endstamp = null,
+                              $returnObj = false)
     {
         global $kronolith_shares, $prefs;
 
@@ -735,8 +760,10 @@ class Kronolith {
             $year = date('Y', $startstamp);
             $day = date('j', $startstamp);
 
-            $endstamp = mktime(0, 0, 0, $month,
-                               $day + $prefs->getValue('freebusy_days'), $year);
+            $endstamp = mktime(0, 0, 0,
+                               $month,
+                               $day + $prefs->getValue('freebusy_days'),
+                               $year);
         }
 
         // Get the Identity for the owner of the share.
@@ -776,8 +803,8 @@ class Kronolith {
             foreach ($events as $event) {
                 $duration = $event->end->timestamp() - $event->start->timestamp();
 
-                // Make sure that we're using the current date for
-                // recurring events.
+                // Make sure that we're using the current date for recurring
+                // events.
                 if (!$event->hasRecurType(KRONOLITH_RECUR_NONE)) {
                     $startThisDay = mktime($event->start->hour,
                                            $event->start->min,
@@ -806,17 +833,17 @@ class Kronolith {
     }
 
     /**
-     * Retrieves the free/busy information for a given email address,
-     * if any information is available.
+     * Retrieves the free/busy information for a given email address, if any
+     * information is available.
      *
-     * @params String   $email  The email address to look for.
+     * @param string $email  The email address to look for.
      *
-     * @return resource   Horde_iCalendar_vfreebusy on success
-     *                    PEAR_Error on failure
+     * @return Horde_iCalendar_vfreebusy  Free/busy component on success,
+     *                                    PEAR_Error on failure
      */
     function getFreeBusy($email)
     {
-        global $prefs;
+        global $conf, $prefs;
 
         require_once 'Horde/iCalendar.php';
         require_once 'Mail/RFC822.php';
@@ -828,7 +855,8 @@ class Kronolith {
             $rfc822 = &new Mail_RFC822();
         }
 
-        $res = $rfc822->parseAddressList($email);
+        $default_domain = empty($conf['storage']['default_domain']) ? null : $conf['storage']['default_domain'];
+        $res = $rfc822->parseAddressList($email, $default_domain);
         if (is_a('PEAR_Error', $res)) {
             return $res;
         }
@@ -838,8 +866,8 @@ class Kronolith {
 
         $email = MIME::rfc822WriteAddress($res[0]->mailbox, $res[0]->host);
 
-        // Check if we can retrieve a VFB from the Free/Busy URL, if
-        // one is set.
+        // Check if we can retrieve a VFB from the Free/Busy URL, if one is
+        // set.
         $url = Kronolith::getFreeBusyUrl($email);
         if (is_a($url, 'PEAR_Error')) {
             return $url;
@@ -876,13 +904,11 @@ class Kronolith {
     }
 
     /**
-     * Search address books for the freebusy URL for a given email
-     * address.
+     * Searchs address books for the freebusy URL for a given email address.
      *
-     * @params String   $email  The email address to look for.
+     * @param string $email  The email address to look for.
      *
-     * @return Mixed    (string) The url on success
-     *                  (boolean) False on failure
+     * @return mixed  The url on success or false on failure.
      */
     function getFreeBusyUrl($email)
     {
@@ -898,14 +924,16 @@ class Kronolith {
      * event. Can be used to send event invitations, event updates as well
      * as event cancellations.
      *
-     * @params object   $event         The event in question.
-     * @params object   $notification  A notification object used to show
-     *                                 result status.
-     * @params integer  $action        The type of notification to send.
-     *                                 One of the KRONOLITH_ITIP_XXX values.
+     * @param Kronolith_Event $event         The event in question.
+     * @param Notification    $notification  A notification object used to show
+     *                                       result status.
+     * @param integer         $action        The type of notification to send.
+     *                                       One of the KRONOLITH_ITIP_XXX values.
      */
     function sendITipNotifications(&$event, &$notification, $action)
     {
+        global $conf;
+
         $attendees = $event->getAttendees();
         /* Exit early. */
         if (!count($attendees)) {
@@ -928,9 +956,17 @@ class Kronolith {
         $myemail = explode('@', $myemail);
         $from = MIME::rfc822WriteAddress($myemail[0], isset($myemail[1]) ? $myemail[1] : '', $ident->getValue('fullname'));
 
+        $mail_driver = $conf['mailer']['type'];
+        $mail_params = $conf['mailer']['params'];
+        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
+            empty($mail_params['username'])) {
+            $mail_params['username'] = Auth::getAuth();
+            $mail_params['password'] = Auth::getCredential('password');
+        }
+
         foreach ($attendees as $email => $status) {
-            // Don't bother sending an invitation/update if the recipient
-            // does not need to participate, or has declined participating
+            // Don't bother sending an invitation/update if the recipient does
+            // not need to participate, or has declined participating
             if ($status['attendance'] == KRONOLITH_PART_NONE ||
                 $status['response'] == KRONOLITH_RESPONSE_DECLINED) continue;
 
@@ -965,7 +1001,7 @@ class Kronolith {
             }
             $message .= _("Attached is an iCalendar file with more information about the event. If your mail client supports iTip requests you can use this file to easily update your local copy of the event.");
 
-            $mime = &new MIME_Message();
+            $mime = &new MIME_Part('multipart/alternative');
             $body = &new MIME_Part('text/plain', String::wrap($message, 80, "\n"));
             $body->setCharset(NLS::getCharset());
 
@@ -975,16 +1011,14 @@ class Kronolith {
             $iCal = &new Horde_iCalendar();
             $iCal->setAttribute('METHOD', $method);
             $iCal->addComponent($event->toiCalendar($iCal, $ident));
-            $ics = &new MIME_Part(
-                'text/calendar',
-                $iCal->exportvCalendar()
-            );
+            $ics = &new MIME_Part('text/calendar', $iCal->exportvCalendar());
             $ics->setName($filename);
             $ics->setContentTypeParameter('METHOD', $method);
             $ics->setCharset(NLS::getCharset());
 
             $mime->addPart($body);
             $mime->addPart($ics);
+            $mime = &MIME_Message::convertMimePart($mime);
 
             // Build the notification headers.
             $msg_headers = &new MIME_Headers();
@@ -993,11 +1027,12 @@ class Kronolith {
             $msg_headers->addHeader('Date', date('r'));
             $msg_headers->addHeader('From', $from);
             $msg_headers->addHeader('To', $email);
-            $msg_headers->addHeader('Subject', $subject);
-            $msg_headers->addHeader('User-Agent', 'Kronolith Calendaring System');
+            $msg_headers->addHeader('Subject', MIME::encode($subject, NLS::getCharset()));
+            require_once KRONOLITH_BASE . '/lib/version.php';
+            $msg_headers->addHeader('User-Agent', 'Kronolith ' . KRONOLITH_VERSION);
             $msg_headers->addMIMEHeaders($mime);
 
-            $status = $mime->send($email, $msg_headers);
+            $status = $mime->send($email, $msg_headers, $mail_driver, $mail_params);
             if (!is_a($status, 'PEAR_Error')) {
                 $notification->push(
                     sprintf(_("The event notification to %s was successfully sent."), $email),
@@ -1048,9 +1083,7 @@ class Kronolith {
     }
 
     /**
-     * Build Kronolith's list of menu items.
-     *
-     * @access public
+     * Builds Kronolith's list of menu items.
      */
     function getMenu($returnType = 'object')
     {
@@ -1060,10 +1093,10 @@ class Kronolith {
             Horde::addScriptFile('goto.inc', 'kronolith');
         }
 
-        // Check here for guest calendars so that we don't get
-        // multiple messages after redirects, etc.
+        // Check here for guest calendars so that we don't get multiple
+        // messages after redirects, etc.
         if (!Auth::getAuth() && !count($GLOBALS['all_calendars'])) {
-            $notification->push(_("No calendars are available to guests."));
+            $GLOBALS['notification']->push(_("No calendars are available to guests."));
         }
 
         require_once 'Horde/Menu.php';
@@ -1108,7 +1141,12 @@ class Kronolith {
      */
     function _sortEventStartTime($a, $b)
     {
-        return (int)date('Gis', $a->start->timestamp()) - (int)date('Gis', $b->start->timestamp());
+        $diff = (int)date('Gis', $a->start->timestamp()) - (int)date('Gis', $b->start->timestamp());
+        if ($diff == 0) {
+            return strcoll($a->title, $b->title);
+        } else {
+            return $diff;
+        }
     }
 
 }
